from marisa_trie import Trie
import re


class TrieSearch(Trie):
    def __init__(self, patterns=None, filepath=None, splitter=u' '):
        super(TrieSearch, self).__init__(patterns)
        self.splitter = splitter
        if filepath:
            self.load(filepath)

    def search_all_patterns(self, text):
        text_idx = 0
        for line in re.split(ur'[\n\r]', text):
            if self.splitter:
                words = re.split(self.splitter, line)
            else:
                words = line
            line_idx = 0
            for i, w in enumerate(words):
                for pattern in self.__search_prefix_patterns(w, words[i + 1:]):
                    yield pattern, text_idx + line_idx
                line_idx += len(w) + len(self.splitter)
            text_idx += line_idx

    def search_longest_patterns(self, text):
        all_patterns = self.search_all_patterns(text)
        check_field = [0] * len(text)
        for pattern, start_idx in sorted(
                all_patterns, key=lambda x: len(x[0]), reverse=True):
            target_field = check_field[start_idx:start_idx + len(pattern)]
            check_sum = sum(target_field)
            if check_sum != len(target_field):
                for i in range(len(pattern)):
                    check_field[start_idx + i] = 1
                yield pattern, start_idx

    def __search_prefix_patterns(self, query, remaining_words):
        if query in self:
            yield query
        # generate next query
        if len(remaining_words):
            next_query = self.splitter.join((query, remaining_words[0]))
            if self.has_keys_with_prefix(next_query):
                for ptn in self.__search_prefix_patterns(
                        next_query, remaining_words[1:]):
                    yield ptn



#!/usr/bin/env python
# coding: utf-8
from __future__ import absolute_import
from __future__ import unicode_literals

from setuptools import setup, find_packages


def _requires_from_file(filename):
    return open(filename).read().splitlines()


try:
    with open('README.rst') as f:
        readme = f.read()
except IOError:
    readme = ''

setup(
    name="trie-search",
    version='0.2.1',
    url='https://github.com/nkmrtty/trie-search',
    author='Tatsuya Nakamura',
    author_email='nkmrtty.com@gmail.com',
    maintainer='Tatsuya Nakamura',
    maintainer_email='nkmrtty.com@gmail.com',
    description='Trie-search is a package for text pattern search using marisa-trie',
    long_description=readme,
    packages=find_packages(),
    install_requires=_requires_from_file('requirements.txt'),
    license="MIT",
    classifiers=[
        'Intended Audience :: Science/Research',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'License :: OSI Approved :: MIT License',
        'Topic :: Text Processing',
    ], )



from marisa_trie import RecordTrie
from .trie import TrieSearch


class RecordTrieSearch(RecordTrie, TrieSearch):
    def __init__(self,
                 record_format,
                 records=None,
                 filepath=None,
                 splitter=u' '):
        super(RecordTrieSearch, self).__init__(record_format, records)
        self.splitter = splitter
        if filepath:
            self.load(filepath)

    def search_all_patterns(self, text, min_weight=0.0):
        for pattern, start_idx in super(RecordTrie,
                                        self).search_all_patterns(text):
            weight = self[pattern][0][0]
            if weight < min_weight:
                continue
            yield pattern, start_idx, weight

    def search_longest_patterns(self, text, min_weight=0.0):
        all_patterns = self.search_all_patterns(text, min_weight)
        check_field = [0] * len(text)
        for pattern, start_idx, weight in sorted(
                all_patterns, key=lambda x: len(x[0]), reverse=True):
            target_field = check_field[start_idx:start_idx + len(pattern)]
            check_sum = sum(target_field)
            if check_sum != len(target_field):
                for i in range(len(pattern)):
                    check_field[start_idx + i] = 1
                yield pattern, start_idx, weight
